package com.dayou.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.dayou.dto.CommissionDeclareDTO;
import com.dayou.dto.UserShareRate;
import com.dayou.entity.CommissionDeclare;
import com.dayou.entity.MajorProductionAllot;
import com.dayou.enums.MainBusinessEnum;
import com.dayou.enums.UserTypeEnum;
import com.dayou.mapper.CommissionDeclareMapper;
import com.dayou.service.ICommissionDeclareService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.service.IMajorProductionAllotService;
import com.dayou.service.workflow.IWorkFlowService;
import com.dayou.utils.LoginContext;
import com.dayou.vo.CommissionDeclareVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;


/**
 * <p>
 * 提成申报 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2024-05-15
 */
@Service
public class CommissionDeclareServiceImpl extends ServiceImpl<CommissionDeclareMapper, CommissionDeclare> implements ICommissionDeclareService {


    @Autowired
    private CommissionDeclareMapper commissionDeclareMapper;

    @Autowired
    private IMajorProductionAllotService majorProductionAllotService;

    @Autowired
    private IWorkFlowService workFlowService;



    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Override
    public Page<CommissionDeclareVO> majorMarketDeclarePage(Page page, CommissionDeclareVO commissionDeclare) {
        Long currentMarketUserId = LoginContext.getCurrentUserId();
        return commissionDeclareMapper.majorMarketDeclarePage(page,commissionDeclare,currentMarketUserId);
    }

    @Override
    public Page<CommissionDeclareVO> majorEvaluateDeclarePage(Page page, CommissionDeclareVO commissionDeclare) {
        Long currentEvaluateUserId = LoginContext.getCurrentUserId();
        return commissionDeclareMapper.majorEvaluateDeclarePage(page,commissionDeclare,currentEvaluateUserId);
    }

    @Override
    @Transactional
    public Boolean commissionDeclare(CommissionDeclareDTO declareDTO) {
        //保存申报记录
        CommissionDeclare commissionDeclare = BeanUtil.copyProperties(declareDTO, CommissionDeclare.class);
        commissionDeclare.setDeclareUserId(LoginContext.getCurrentUserId());
        this.save(commissionDeclare);
        //设置提成比例
        List<UserShareRate> userShareRates = declareDTO.getUserShareRates();
        List<MajorProductionAllot> collect = userShareRates.stream().map(x -> {
            MajorProductionAllot majorProductionAllot = new MajorProductionAllot();
            majorProductionAllot.setMajorId(declareDTO.getBusinessId());
            majorProductionAllot.setMajorProductionId(declareDTO.getProductionId());
            majorProductionAllot.setUserId(x.getUserId());
            majorProductionAllot.setRatio(x.getRate());
            majorProductionAllot.setUserType(UserTypeEnum.MARKET.name());
            return majorProductionAllot;
        }).collect(Collectors.toList());
        majorProductionAllotService.saveBatch(collect);
        //提交审核流程

        Long businessId = declareDTO.getBusinessId();
        Long productionId = declareDTO.getProductionId();
        if (productionId==null){
            productionId = businessId;
        }
        workFlowService.openingDeclare(MainBusinessEnum.COMMISSION_DECLARE_MAJOR_MARKET,productionId);
        return null;
    }
}
